#define	VERSION	"1.0b4"
/*
**	xdiff -- graphical diff for X
**	main.c: main implementation
**
**	Copyright (C) 1996  Paul J. Lucas
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 1, or (at your option)
**	any later version.
** 
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
** 
**	You should have received a copy of the GNU General Public License
**	along with this program; if not, write to the Free Software
**	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* system */
#include <stdio.h>
#include <stdlib.h>				/* for exit() */
#include <string.h>				/* for str...() */
#include <sys/types.h>
#include <unistd.h>				/* for dup(), fork(), ... */

/* X/Motif */
#include <X11/Intrinsic.h>

/* local */
#include "gui.h"
#include "util.h"

/* external global variables */
extern int		errno;

/* external functions */
extern int /* bool */	read_files PJL_PROTO(( char const *file_name[] ));
extern void		text_output PJL_PROTO(());

/* global variable definitions */
char const		*me;			 /* our name */

/* local functions */
static void		fork_diff PJL_PROTO((
				char const *diff_opts, char const *file_name[]
			));

/*************************/
	int
main PJL_ARG_LIST(( argc, argv ))
	PJL_ARG_DEF( int argc )
	PJL_ARG_END( char const *argv[] )
{
	extern char *optarg;
	extern int optind, opterr;
	int opt;				/* command-line option */
	int exit_opt = 0 /* false */;
	int text_opt = 0 /* false */;
	static char diff_opts[5] = "-";		/* string for diff options */

	/* strip pathname from argv[0] */
	if ( me = strrchr( argv[0], '/' ) )
		++me;
	else
		me = argv[0];

	/*
	** Even though I don't want to initialize X at this point, I have to in
	** order to allow X to parse out X-specific options.
	*/
	init_gui( &argc, argv );

	/*
	** Now parse our command-line options.
	**
	** If you change the options, remember to change the usage message in
	** usage() defined in util.c!
	*/
	opterr = 1;
	while ( ( opt = getopt( argc, (char**)argv, "bEiTvw" ) ) != EOF )
		switch ( opt ) {
			case 'b':
			case 'i':
			case 'w':
				/* these are all diff options */
				if ( !strchr( diff_opts, opt ) ) {
					/* if we didn't add it before... */
					static char opt_string[2];
					*opt_string = (char)opt;
					strcat( diff_opts, opt_string );
				}
				break;
			case 'E':
				exit_opt = 1 /* true */;
				break;
			case 'T':
				text_opt = 1 /* true */;
				break;
			case 'v':
				goto version;
			case '?':
				usage();
		}
	argc -= optind, argv += optind;
	if ( argc != 2 )
		usage();

	fork_diff( diff_opts[1] ? diff_opts : 0, argv );

	if ( !read_files( argv ) && exit_opt ) {
		/*
		** The files are the same and the user specified the "exit if
		** same" option, so exit.
		*/
		exit( 0 );
	}

	if ( text_opt )
		text_output();
	else
		gui_output();

	return 0;

version:
	fprintf( stderr, "%s: version %s\n", me, VERSION );
	exit( 0 );
}

/*************************/
	static void
fork_diff PJL_ARG_LIST(( diff_opts, file_name ))
	PJL_ARG_DEF( char const *diff_opts )
	PJL_ARG_END( char const *file_name[] )
{
	int pipe_fd[2];
	pid_t diff_pid;

	/*
	** Create a pipe to connect our diff child process's stdout to our
	** stdin.
	*/
	if ( pipe( pipe_fd ) == -1 )
		my_perror( "pipe()" );

	if ( ( diff_pid = fork() ) == -1 )
		my_perror( "fork()" );

	if ( diff_pid == 0 ) {
		/*
		** The diff child process -- connect our stdout to our parent's
		** stdin.
		*/
		XtDestroyApplicationContext( app );
		close( pipe_fd[0] );
		close( 1 );
		if ( dup( pipe_fd[1] ) == -1 )
			my_perror( "dup()" );
		close( 0 );
		close( pipe_fd[1] );

		if ( diff_opts )
			execlp( "diff", "diff",
				diff_opts, file_name[0], file_name[1], (char*)0
			);
		else
			execlp( "diff", "diff",
				file_name[0], file_name[1], (char*)0
			);

		/* we should never get here */
		my_perror( "execlp()" );
	}

	/*
	** The original parent process -- connect our stdin to the diff child
	** process's stdout.
	*/
	close( 0 );
	close( pipe_fd[1] );
	if ( dup( pipe_fd[0] ) == -1 )
		my_perror( "dup()" );
	close( pipe_fd[0] );
}
