/*
**	xdiff -- graphical diff for X
**	util.c: utility routines implementation
**
**	Copyright (C) 1996  Paul J. Lucas
**
**	This program is free software; you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation; either version 1, or (at your option)
**	any later version.
** 
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
** 
**	You should have received a copy of the GNU General Public License
**	along with this program; if not, write to the Free Software
**	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* system */
#include <stdio.h>
#include <stdlib.h>				/* for exit() */

/* local */
#include "c_compat.h"

#define	TAB_EQUAL	8			/* 1 tab = 8 spaces */

/* external global variables */
extern int		errno;
extern char const	*me;

/*************************/
	int
expand_tabs PJL_ARG_LIST(( from, to, buf_size ))
	PJL_ARG_DEF( register char const *from )
	PJL_ARG_DEF( register char *to )
	PJL_ARG_END( int buf_size )
{
	/*
	** Expand all tabs to the right number of spaces.  Return the new
	** string length.
	*/
	register int col = 0;

	while (	*from && *from != '\n' && col < buf_size ) {
		if ( *from == '\t' ) {
			register int n = TAB_EQUAL - col % TAB_EQUAL;
			col += n;
			while ( n-- )
				*to++ = ' ';
		} else {
			*to++ = *from;
			++col;
		}
		++from;
	}
	*to = '\0';

	return col;
}

/*************************/
	void
my_perror PJL_ARG_LIST(( msg ))
	PJL_ARG_END( char const *msg )
{
	fprintf( stderr, "%s: ", me );
	perror( msg );
	exit( errno );
}

/*************************/
	void
usage()
{
	fprintf( stderr, "usage: %s [-bEiTvw] file1 file2\n", me );
	exit( 2 );
}
